# coding=iso-8859-15
#
# Newsflash Python extension widget system.
#
# Copyright (c) Sami Visnen 2008
#
import xml.sax.saxutils
import xml.sax.handler
import xml.sax

LISTBOX_SINGLE_SELECT = 1
LISTBOX_MULTI_SELECT  = 2

MAX_WIDTH  = 'max'
MAX_HEIGHT = 'max'

def xmlesc(string):
    if string==None:
        return ''
    
    dic={"'" : "&apos;", '"' : "&quot;"}
    ret = xml.sax.saxutils.escape(string, dic)

    # there's no Unicode support yet in the host application
    # so we gotta use ext ascii for now.
    if isinstance(ret, unicode)==True:
        ret = ret.encode('iso-8859-15', 'ignore')

    return ret

def funcname(func):
    if func==None:
        return ''
    return func.__name__


class Window(xml.sax.handler.ContentHandler):
    """Window is the container class for widgets."""
    def __init__(self, widgets):
        self.widgets = widgets

    def __getitem__(self, index):
        return self.widgets[index]

    def marshal(self):
        """Serialiaze this structure into an internal representation for the host application."""
        lst = ['<?xml version="1.0" encoding="ISO-8859-1"?>\n']
        lst.append('<window>\n')
        # marshal the widgets
        if self.widgets!=None:
            for widget in self.widgets:
                lst.append(widget.marshal())
                lst.append('\n')
                
        lst.append('</window>\n')
        return ''.join(lst)

        # return str(''.join(lst))
        #return unicode(''.join(lst)).encode('iso-8859-15')
        #foo = u''.join(lst)
        #return foo.encode('iso-8859-15')

    def update(self, XML):
        """Update this structure from the internal representation."""
        xml.sax.parseString(XML, self)
        pass


    def startElement(self, name, attribs):
        if name == 'window':
            # todo: grab some window props here
            return
        # find the widget we're updating
        wid = Window.find(self, int(attribs["id"]))        
        if name == 'input':
            wid.value = attribs['value']
        elif name == 'checkbox':
            wid.check = bool(int(attribs['check']))

    def endElement(self, name):
        pass
    
    def find(self, wid):
        for widget in self.widgets:
            if hasattr(widget, 'id') and widget.id == wid:
                return widget
            else:
                if id(widget) == wid:
                    return widget
        return None
        
class Button:
    """Button represents a push button."""
    def __init__(self, x, y, str, on_click=None):
        """Create new button."""
        self.x   = x
        self.y   = y
        self.str = str
        self.on_click = on_click

    def marshal(self):
        return '<button id="%d" x="%d" y="%d" text="%s" on_click="%s"/>' % (
            id(self), self.x, self.y, xmlesc(self.str),funcname(self.on_click))


class List:
    """Listbox shows a list of items in a listlike manner."""
    def __init__(self, x, y, width, height, items,  selmode=LISTBOX_SINGLE_SELECT, on_select=None ):
        self.x         = x
        self.y         = y
        self.width     = width
        self.height    = height
        self.items     = items
        self.selmode   = selmode
        self.scroll    = 0
        # Selection key press (typically either Enter or Space)
        self.on_select = on_select

    def __getitem__(self, index):
        return self.items[index]

    def marshal(self):
        lst = ['<list id="%d" x="%d" y="%d" width="%s" height="%s" mode=\"%d\" scroll="%d" on_select="%s">\n' %
               (id(self), self.x, self.y, self.width, self.height, self.selmode, self.scroll, funcname(self.on_select))
               ]
        if self.items != None:
            for item in self.items:
                lst.append('<item>%s</item>\n' % xmlesc(item))
        lst.append('</list>')
        return ''.join(lst)


class Text:
    """Text is for dispalying a static string of text."""
    def __init__(self, x, y, str):
        self.x   = x
        self.y   = y
        self.str = str

    def marshal(self):
        return '<text id="%d" x="%d" y="%d" text="%s"/>' % (id(self), self.x, self.y, xmlesc(self.str))

class Input:
    """Input takes user input from the keyboard."""
    def __init__(self, x, y, width, val=None, on_enter=None):
        self.x        = x
        self.y        = y
        self.width    = width
        self.value    = val
        # Enter key press
        self.on_enter = on_enter
        self.id       = id(self)

    def marshal(self):
        return '<input id="%d" x="%d" y="%d" width="%s" value="%s" on_enter="%s"/>' % (
            self.id, self.x, self.y, self.width, xmlesc(self.value), funcname(self.on_enter))

class View:
    """Displays multiple rows of static text."""
    def __init__(self, x, y, width, height, data=None):
        """Construct a new view object with initial data.
        Data should simply be a string with each line separated
        by new a newline character.
        """
        self.x = x
        self.y = y
        self.width  = width
        self.height = height
        self.data   = data

    def marshal(self):
        xml = '<view id="%d" x="%d" y="%d" width="%s" height="%s">' % (
            id(self), self.x, self.y, self.width, self.height)
        if self.data != None:
            xml += xmlesc(self.data)
        xml += '</view>'
        return xml

class Checkbox:
    """Checkbox."""
    def __init__(self, x, y, text, check=False, on_click=None):
        self.x = x
        self.y = y
        self.text = text
        self.check = check
        self.on_click = on_click
        self.id = id(self)

    def marshal(self):
        xml = '<checkbox id="%d" x="%d" y="%d" text="%s" check="%d" on_click="%s"/>' % (
            self.id, self.x, self.y, xmlesc(self.text), int(self.check), funcname(self.on_click))
        return xml

class Progress:
    """Progress bar."""
    def __init__(self, x, y, width, lo, hi, pos, text=None):
        self.x = x
        self.y = y
        self.width = width
        self.lo = lo
        self.hi = hi
        self.pos = pos
        self.text = text

    def marshal(self):
        xml = '<progress id="%d" x="%d" y="%d" width="%s" lo="%d" hi="%d" pos="%d" text="%s"/>' % (
            id(self), self.x, self.y, self.width, self.lo, self.hi, self.pos, xmlesc(self.text))
        return xml
        
def test():
    wnd = Window([
        Input(0, 0, "max", 'a string', None),
        View(1, 1, "max", "max", "Here's some string data for the view")
        ])
    wnd[0].id = 123
    xml = wnd.marshal()
    print xml


